(ns com.aurellem.exp.item-bridge
  (:use (com.aurellem.gb util constants gb-driver vbm items assembly))
  (:use (com.aurellem.run title save-corruption))
  (:use (com.aurellem.exp pokemon))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(defn corrupt-item-state []
  (second (destroy-item-end-of-list-marker)))

(defn corrupt-item-state []
  (read-state "corrupt-items"))

(defn view-memory-range [state start end]
  (dorun
   (map (fn [loc val]
          (println (format "%04X : %02X" loc val)))
        (range start end) (subvec (vec (memory state)) start end)))
  state)

(defn almost-broken
  "if one more memory location is turned into 0x03, the game crashes."
  [n]
  (view-memory-range
   (set-inv-mem (mid-game)
                (concat [0xFF] (repeat 64 0x03)
                        (subvec (vec (memory (mid-game)))
                                (+ item-list-start 65)
                                (+ item-list-start 65 n))
                        (repeat (- 255 65 n) 0x03)))
   item-list-start (+ item-list-start 255)))

(defn actually-broken
  "if this memory location is turned into 0x03, the game crashes."
  []
  (set-memory (mid-game) 0xD35D 0x03))


;; (almost-broken 20) more or less works

(defn capture-program-counter
  "records the program counter for each tick"
  [^SaveState state ticks]
  (let [i (atom 0)]
    (reduce (fn [[program-counters state] _]
              (println (swap! i inc))
               [(conj program-counters (PC state))
                (tick state)])
              [[] state]
              (range ticks))))


(defn capture-program-counter
  [^SaveState state ticks]
  (set-state! state)
  (loop [i 0
         pcs []]
    (if (= i ticks)
      pcs
      (do 
        (com.aurellem.gb.Gb/tick)
        (recur (inc i)
               (conj pcs (first (registers))))))))

(defn loop-program []
  [0xF3     ;0xD31D  ;; disable-interrupts

   0xC3     ;; loop forever
   0x1D
   0xD3])

(def map-function-address-start 0xD36D)

(defn test-loop []
  (capture-program-counter
   (-> (mid-game) 
       (set-memory-range 0xD31D (loop-program))
       (set-memory-range
        map-function-address-start
        [0xD3 0x1D])
       (IE! 0))
   1000))


(defn-memo corrupt-moves []
  (concat
   (first
    (->>
     [[] (mid-game)]
     (advance [:b] [:b :start])
     (advance [] [:d])
     (play-moves [[] [] [] [:d] [] [] [] [:d] [] [] [:a]])
     scroll-text
     (play-moves
      ;; this section is copied from speedrun-2942
      ;; and corrupts the save so that the end-of-list marker
      ;; for the pokemon roster is destroyed, but the save is still
      ;; playable.
      [[] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
       [] [] [] [] [] [] [] [] [] [] [:select] [:restart]])
     (play-moves
      (first (title)))
     (advance [] [:start])
     (advance [] [:a])
     (advance [:a] [:a :start])))
   [[]]))
  


(defn corrupt
  "enter the codes to destroy the
   pokemon list using save corruption"
  ([^SaveState state]
  (run-moves
   state
   (corrupt-moves)))
  ([] (corrupt @current-state)))
  
   

             



